using UnityEngine;
using System;
using System.Runtime.InteropServices;

[RequireComponent (typeof (MeshCollider))]
[RequireComponent (typeof (MeshFilter))]
[RequireComponent (typeof (MeshRenderer))]

class testObj : MonoBehaviour {

   
    [DllImport ("sgCorePlugin")] private static extern void   sgInit();
	[DllImport ("sgCorePlugin")] private static extern void   sgFree();
	[DllImport ("sgCorePlugin")] private static extern void   sgCreateHouseModel();
	[DllImport ("sgCorePlugin")] private static extern IntPtr    sgGetHeadObject();
	[DllImport ("sgCorePlugin")] private static extern IntPtr    sgGetNextObject(IntPtr objPtr);
	[DllImport ("sgCorePlugin")] private static extern int       sgGetObjectsCount();
	
	[DllImport ("sgCorePlugin")] private static extern int      sgGetObjectTrianglesCount(IntPtr  objPtr);
	[DllImport ("sgCorePlugin")] private static extern IntPtr   sgGetObjectMatrix(IntPtr  objPtr);
	[DllImport ("sgCorePlugin")] private static extern IntPtr   sgGetObjectVertexes(IntPtr  objPtr);
	[DllImport ("sgCorePlugin")] private static extern IntPtr   sgGetObjectNormals(IntPtr  objPtr);
    [DllImport ("sgCorePlugin")] private static extern IntPtr   sgGetObjectUVs(IntPtr  objPtr);
	[DllImport ("sgCorePlugin")] private static extern int      sgGetObjectMaterialIndex(IntPtr  objPtr);

	public struct vec3d
    {
	  public double x;
      public double y;
	  public double z;
    }
	
	public struct vec2d
    {
	  public double x;
      public double y;
    }
	
	public struct matr
    {
	  public double  m11, m12, m13, m14, m21, m22, m23, m24, m31, m32, m33, m34, m41, m42, m43, m44;
    }
	
	private  bool   showButton = true;
	
   void Start () {
  	
   }
   
   void Update () {
     
      
   }
	
	Vector3  PointToMatrix(Vector3 pnt, matr matrix)
	{
		Vector3 resltPnt;
		
		resltPnt.x = (float)(pnt.x*matrix.m11 + pnt.y*matrix.m21 + pnt.z*matrix.m31  + matrix.m41);
	    resltPnt.y = (float)(pnt.x*matrix.m12 + pnt.y*matrix.m22 + pnt.z*matrix.m32  + matrix.m42);
	    resltPnt.z = (float)(pnt.x*matrix.m13 + pnt.y*matrix.m23 + pnt.z*matrix.m33 + matrix.m43);
		
		return  resltPnt;
	}
   
   void OnGUI()
   {
      GUILayout.BeginArea(new Rect(10,10, 200, 100));
	  GUILayout.BeginVertical();
	  
	  if (showButton && GUILayout.Button("Generate Model"))
	  {
			sgInit();
			sgCreateHouseModel();
			
			IntPtr  curObjPtr = sgGetHeadObject();
		
			int CurObjInd = 0;
			int objectsCount = sgGetObjectsCount();
			
			while (CurObjInd<objectsCount)
			{
				GameObject go = new GameObject();
				go.AddComponent("MeshFilter");
                go.AddComponent("MeshRenderer");
		
				MeshFilter meshFilter = go.GetComponent<MeshFilter>();
				if (meshFilter==null){
					Debug.LogError("MeshFilter not found!");
					return;
				}
				
				int vCnt = sgGetObjectTrianglesCount(curObjPtr)*3;
								
				Vector3[]  allVerts = new Vector3[vCnt];
				Vector3[]  allNormals = new Vector3[vCnt];

				int[]      trIndexes = new int[vCnt];
				Vector2[]  uvs = new Vector2[vCnt];
				
				IntPtr vertVals = sgGetObjectVertexes(curObjPtr);
				IntPtr normVals = sgGetObjectNormals(curObjPtr);
				IntPtr uvVals =   sgGetObjectUVs(curObjPtr);
				IntPtr objMatrPtr =  sgGetObjectMatrix(curObjPtr);
				
				int structSize3D = Marshal.SizeOf(typeof(vec3d));
				int structSize2D = Marshal.SizeOf(typeof(vec2d));
				
				matr objMatrix = (matr)Marshal.PtrToStructure(objMatrPtr, typeof(matr));
							
				for (int i=0, j=0; i<vCnt; i++)
				{
					IntPtr data = new IntPtr(vertVals.ToInt64() + structSize3D * i);
                    vec3d pnt = (vec3d)Marshal.PtrToStructure(data, typeof(vec3d));
					allVerts[j] = new Vector3((float)pnt.x, (float)pnt.y, (float)pnt.z);
					allVerts[j] = PointToMatrix(allVerts[j], objMatrix);
					

					data = new IntPtr(normVals.ToInt64() + structSize3D * i);
                    pnt = (vec3d)Marshal.PtrToStructure(data, typeof(vec3d));
					allNormals[j] = new Vector3((float)pnt.x, (float)pnt.y, (float)pnt.z);
					
					data = new IntPtr(uvVals.ToInt64() + structSize2D * i);
                    vec2d tmpUV = (vec2d)Marshal.PtrToStructure(data, typeof(vec2d));
					uvs[j] = new Vector2((float)tmpUV.x/1000, (float)tmpUV.y/1000);
										
					j++;
										
					trIndexes[i] = i;
				}
				
				Mesh mesh = meshFilter.sharedMesh;
				if (mesh == null){
					meshFilter.mesh = new Mesh();
					mesh = meshFilter.sharedMesh;
				}
				mesh.Clear();
				
				mesh.vertices = allVerts;
				mesh.normals = allNormals;
				mesh.triangles = trIndexes;
				mesh.uv = uvs;  
				
				//mesh.RecalculateNormals();
				mesh.RecalculateBounds();
				mesh.Optimize();
				
				if (sgGetObjectMaterialIndex(curObjPtr)==1)
 				 go.renderer.material = Resources.Load("roofMat", typeof(Material)) as Material;
				else
				  go.renderer.material = Resources.Load("woodMat", typeof(Material)) as Material;
						
				curObjPtr = sgGetNextObject(curObjPtr);
				CurObjInd++;
			}
	     			
			sgFree();
			
			showButton = false;
	  }
	  
	  GUILayout.EndVertical();
	  GUILayout.EndArea();
   }
} 
